package com.graphaware.neo4j.representation.property;

/**
 * Interface for (typically immutable) components that contain some properties and are able to return a copy
 * (again immutable) of themselves, with a single property omitted.
 */
public interface MakesCopyWithoutProperty<T extends MakesCopyWithoutProperty<T>> {

    /**
     * Make a copy of this instance with a single key-value pair missing.
     *
     * @param key to omit in the copy. If the key does not exist in the representation, an identical copy will be returned.
     * @return copy of this instance except one key-value pair.
     */
    public T without(String key);
}
