package com.graphaware.neo4j.representation.relationship;

import com.graphaware.neo4j.representation.property.SimpleProperties;
import org.neo4j.graphdb.Direction;
import org.neo4j.graphdb.Node;
import org.neo4j.graphdb.PropertyContainer;
import org.neo4j.graphdb.RelationshipType;

import java.util.Map;

/**
 * Simple {@link Relationship}. Provides to/from String conversion and access to the represented type, direction, and properties.
 */
public class SimpleRelationship extends StringConvertibleRelationship<SimpleProperties> implements Relationship<SimpleProperties> {

    /**
     * Construct a relationship representation.
     *
     * @param relationship Neo4j relationship to represent.
     * @param pointOfView  node which is looking at this relationship and thus determines its direction.
     */
    public SimpleRelationship(org.neo4j.graphdb.Relationship relationship, Node pointOfView) {
        super(relationship, pointOfView);
    }

    /**
     * Construct a relationship representation. Please note that using this constructor, the actual properties on the
     * relationship are ignored! The provided properties are used instead.
     *
     * @param relationship Neo4j relationship to represent.
     * @param pointOfView  node which is looking at this relationship and thus determines its direction.
     * @param properties   to use as if they were in the relationship.
     */
    public SimpleRelationship(org.neo4j.graphdb.Relationship relationship, Node pointOfView, SimpleProperties properties) {
        super(relationship, pointOfView, properties);
    }

    /**
     * Construct a relationship representation with no properties.
     *
     * @param type       type.
     * @param direction  direction.
     */
    public SimpleRelationship(RelationshipType type, Direction direction) {
        super(type, direction);
    }

    /**
     * Construct a relationship representation.
     *
     * @param type       type.
     * @param direction  direction.
     * @param properties props.
     */
    public SimpleRelationship(RelationshipType type, Direction direction, SimpleProperties properties) {
        super(type, direction, properties);
    }

    /**
     * Construct a relationship representation.
     *
     * @param type       type.
     * @param direction  direction.
     * @param properties props.
     */
    public SimpleRelationship(RelationshipType type, Direction direction, Map<String, String> properties) {
        super(type, direction, properties);
    }

    /**
     * Construct a relationship representation from a string.
     *
     * @param string string to construct relationship from. Must be of the form type#direction#key1#value1#key2#value2...
     *               (assuming the default {@link com.graphaware.neo4j.utils.Constants#SEPARATOR}.
     */
    public SimpleRelationship(String string) {
        super(string);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected SimpleProperties newProperties(String string) {
        return new SimpleProperties(string);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected SimpleProperties newProperties(PropertyContainer propertyContainer) {
        return new SimpleProperties(propertyContainer);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected SimpleProperties newProperties(Map<String, String> properties) {
        return new SimpleProperties(properties);
    }
}
