package com.graphaware.neo4j.utils.relationship;

import org.apache.log4j.Logger;
import org.neo4j.graphdb.Direction;
import org.neo4j.graphdb.Node;
import org.neo4j.graphdb.Relationship;

/**
 * Utility class for static methods related to {@link Relationship} {@link Direction}.
 */
public final class DirectionUtils {
    private static final Logger LOG = Logger.getLogger(DirectionUtils.class);

    private DirectionUtils() {
    }

    /**
     * Resolve the direction of a relationship from a pointOfView's point of view. Relationships to self are resolved as
     * {@link Direction#OUTGOING} by convention.
     *
     * @param relationship to resolve direction for.
     * @param pointOfView  direction will be from this pointOfView's point of view.
     * @return direction of the relationship from the given pointOfView's point of view.
     */
    public static Direction resolveDirection(Relationship relationship, Node pointOfView) {
        if (relationship.getEndNode().getId() != pointOfView.getId() && relationship.getStartNode().getId() != pointOfView.getId()) {
            String message = "Provided relationship (" + relationship.getId() + ") does not have node (" + pointOfView.getId() + ") on either of its ends!";
            LOG.warn(message);
            throw new IllegalArgumentException(message);
        }

        return ((relationship.getStartNode().getId() == pointOfView.getId()) ? Direction.OUTGOING : Direction.INCOMING);
    }
}
