package com.graphaware.neo4j.utils.tx.mutate;

import org.neo4j.graphdb.GraphDatabaseService;
import org.neo4j.graphdb.Node;
import org.neo4j.graphdb.Relationship;
import org.neo4j.graphdb.RelationshipType;

import java.util.HashMap;
import java.util.Map;

/**
 *  A "fake" relationship that maintains a list of properties separately to the actual real relationship.
 */
public class FakeRelationship implements Relationship {

    private final Relationship realRelationship;
    private final Map<String, Object> properties = new HashMap<String, Object>();

    public FakeRelationship(Relationship realRelationship) {
        this.realRelationship = realRelationship;
    }

    @Override
    public long getId() {
        return realRelationship.getId();
    }

    @Override
    public void delete() {
        realRelationship.delete();
    }

    @Override
    public Node getStartNode() {
        return realRelationship.getStartNode();
    }

    @Override
    public Node getEndNode() {
        return realRelationship.getEndNode();
    }

    @Override
    public Node getOtherNode(Node node) {
        return realRelationship.getOtherNode(node);
    }

    @Override
    public Node[] getNodes() {
        return realRelationship.getNodes();
    }

    @Override
    public RelationshipType getType() {
        return realRelationship.getType();
    }

    @Override
    public boolean isType(RelationshipType type) {
        return realRelationship.isType(type);
    }

    @Override
    public GraphDatabaseService getGraphDatabase() {
        return realRelationship.getGraphDatabase();
    }

    @Override
    public boolean hasProperty(String key) {
        return properties.containsKey(key);
    }

    @Override
    public Object getProperty(String key) {
        return properties.get(key);
    }

    @Override
    public Object getProperty(String key, Object defaultValue) {
        if (getProperty(key) == null) {
            return defaultValue;
        }
        return getProperty(key);
    }

    @Override
    public void setProperty(String key, Object value) {
        properties.put(key, value);
    }

    @Override
    public Object removeProperty(String key) {
        return properties.remove(key);
    }

    @Override
    public Iterable<String> getPropertyKeys() {
        return properties.keySet();
    }

    @Override
    @Deprecated
    public Iterable<Object> getPropertyValues() {
        return properties.values();
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        FakeRelationship that = (FakeRelationship) o;

        if (!properties.equals(that.properties)) return false;
        if (!realRelationship.equals(that.realRelationship)) return false;

        return true;
    }

    @Override
    public int hashCode() {
        int result = realRelationship.hashCode();
        result = 31 * result + properties.hashCode();
        return result;
    }
}
